<?php
/**
 * @file
 * Function called by Ajax methods in search pages
 * 
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

/**
 * Bookmark one or more items
 * 
 * @return (String)
 *   A JSON string containing the result code:
 *   - -1: the bookmark does already exist
 *   - 0: the operation failed
 *   - 1: new bookmark were created
 */
function xc_search_ajax_bookmark_item() {
  global $user;
  
  list($nid, $sid) = explode('|', $_GET['id']);
  $result_code = xc_account_bookmark_item($user->uid, $nid, $sid);
  
  drupal_json(array('result' => (int)$result_code));
}

/**
 * Remove bookmark items
 */
function xc_search_ajax_remove_bookmark_item() {
  global $user;
  
  list($nid, $sid) = explode('|', $_GET['id']);
  $result_code = xc_account_remove_bookmark_item($user->uid, $nid, $sid);
  
  drupal_json(array('result' => (int)$result_code));
}

function xc_search_ajax_image_info() {
  $isbn = $_GET['ISBN'];
  $upc  = $_GET['UPC'];
  $oclc = $_GET['OCLC'];
  $size = (isset($_GET['large']) && $_GET['large'] == 1) ? SYNDETICS_LARGE : SYNDETICS_SMALL;
  $url = xc_search_get_syndetics_image($isbn, $upc, $oclc, $size);
  drupal_set_header('Content-Type: text/plain; charset: utf-8');
  printf('{"image_url": "%s"}', $url);
}

/**
 * Gets the WorldCat Identity URL, and redirects
 */
function xc_search_ajax_open_worldcat_identity_url() {
  // alternative way, using OpenURL:
  // http://worldcat.org/identities/find
  // ?url_ver=Z39.88-2004
  // &rft_val_fmt=info:ofi/fmt:kev:mtx:identity
  // &rft.namelast=Hickey
  // &rft.namefirst=Thomas
  // &rft_id=info:oclcnum/9746645
  $worldcat_url = 'http://worldcat.org/identities/find?fullName=' . urlencode($_GET['sURL']);
  $content = file_get_contents($worldcat_url);
  $xml = new SimpleXMLElement($content);
  $types = array('ExactMatches', 'FuzzyFirstName', 'FuzzyFamilyName');
  foreach ($types as $type) {
    $results = $xml->xpath('/nameAuthorities/match[@type="' . $type .'"]/uri');
    if (!empty($results)) {
      // an example URI:
      // name with LCCN: '/identities/lccn-n79-6533'
      // name without LCCN: '/identities/np-levan, ralph r'
      $uri = $results[0];
      break;
    }
  }
  $url = !empty($uri) ? 'http://worldcat.org' . $uri : $_GET['sURL'];
  drupal_set_header('Location: ' . $url);
  return;
}

function xc_search_ajax_ncip_info() {
  global $_xc_search_ncip_servers;
  
  $ncip_id  = $_GET['ncip_id'];
  $bib_id   = $_GET['bib_id'];
  $availability = '';
  $call_number =  '';
  $response = xc_ils_get_availability($ncip_id, array('bib' => array($bib_id)));
  if (isset($response['error'])) {
    $availability = '<span class="not-available">' . t('Availability Unknown') . '</span>';
    xc_log_error('ncip', $response['error']['ErrorMessage']);
  }
  else {
    // $bib = $status = $response['results']['bib'][$bib_id][0];
    $available_items = 0;
    foreach ($response['results']['bib'][$bib_id] as $key => $bib) {
      if ($bib['CirculationStatus'] == 'Not Charged') {
        $call_number = $bib['ItemDescription']['CallNumber'];
        $availability = '<span class="available">' . t('Available') . '</span> &mdash; ' . $bib['Location'];
        $available_items++;
      }
    }
    
    if ($available_items == 0) {
      $availability = '<span class="not-available">' . t('Not Available') . '</span>';
      $call_number = t('not available');
    }
    elseif ($available_items > 1) {
      $availability = '<span class="available">' . t('Multiple items available') . '</span>';
      $call_number = t('multiple');
    }
  }
  xc_log_info('xc', $bib_id . ' => ' . $availability);
  
  if (isset($_GET['debug'])) {
    return $availability;
  }
  
  drupal_json(array(
    'status' => 0, 
    'availability' => $availability,
    'call_number' => $call_number,
  ));
}

function xc_search_ajax_ncip_info_full() {
  $ncip_item_ids = array();
  $provider_id = $_GET['provider_id'];
  if ($_GET['bib_ids'] == '') {
    $locations = '';
  }
  else {
    $bib_ids = explode(',', $_GET['bib_ids']);
    $token       = $_GET['token'];
    xc_ajax_bootstrap($token);
    $ncip_item_ids[$provider_id] = $bib_ids;
    $response = xc_search_get_full_ncip($ncip_item_ids);
    $locations = $response['locations'];
    $count = $response['count'];
    // theme('manifestation_full_locations', $locations)
  }
  
  if (isset($_GET['debug'])) {
    return $locations;
  }
  
  drupal_json(array(
    'status' => 0, 
    'content' => $locations,
    'count' => $count
  ));
}

function xc_search_get_full_ncip($ncip_ids) {
  global $_xc_search_ncip_servers;
  $rows = array();
  // TODO: put this value into UI's properties
  $max_rows_to_display = 5;

  $has_holdings_info = FALSE;
  foreach ($ncip_ids as $ncip_provider_id => $ncip_item_ids) {
    if (!is_array($ncip_item_ids)) {
      $ncip_item_ids = array($ncip_item_ids);
    }
    $response = xc_ils_get_availability($ncip_provider_id, array('bib' => $ncip_item_ids));
    if (isset($response['error'])) {
      $rows[] = array($_xc_search_ncip_servers[$ncip_provider_id], 
        t('Availability Unknown'), '');
      $msg = t('Error occured while requesting the circulation status of %item: %msg',
        array(
          '%item' => xc_util_conditional_join(', ', $ncip_item_ids),
          '%msg' => $response['error']['ErrorMessage']
        ));
      xc_log_error('ncip', $response['error']['ErrorMessage']);
    }
    else {
      $i = 0;
      foreach ($ncip_item_ids as $ncip_item_id) {
        if (!isset($response['results']['bib'][$ncip_item_id])) {
          $rows[] = array('data' => array(
            $_xc_search_ncip_servers[$ncip_provider_id],  
            t('Availability Unknown'), 
            '', 
            ''
          ));
        }
        else {
          foreach ($response['results']['bib'][$ncip_item_id] as $item) {
            // TODO: admin interface for xc_search_mapit_file, maybe for ILS settings?
            $path = variable_get('xc_search_mapit_file',
              drupal_get_path('module', 'xc_search') . '/demo_stack_directory.pdf');
            $url = $GLOBALS['base_url'] . '/' . $path;

            if (!empty($item['Location'])) {
              $location = $item['Location'];
            }
            else {
              $location = $_xc_search_ncip_servers[$ncip_provider_id];
            }
            // TODO: inspect this: <span class="">
            $location = '<span class="">' 
              . l($location, $url, array('attributes' => array('target' => '_blank')));

            if (!empty($item['CirculationStatus'])) {
              if ($item['CirculationStatus'] == 'Not Charged') {
                $status = t('Available');
                $status_class = 'xc-circ-status-available';
              }
              else {
                $status = $item['CirculationStatus'];
                $status_class = 'xc-circ-status-other';
              }
            }
            else {
              $status = t('Availability Unknown');
              $status_class = 'xc-circ-status-unknown';
            }

            $call_number = $item['ItemDescription']['CallNumber'];
            $holdings_info = $item['ItemDescription']['HoldingsInformation'];
            if ($has_holdings_info == FALSE && !empty($holdings_info)) {
              $has_holdings_info = TRUE;
            }
            $data = array(
              array('data' => $location,    'class' => 'xc-circ-location'),
              array('data' => $call_number, 'class' => 'xc-circ-call-number'),
              array('data' => $holdings_info, 'class' => 'xc-circ-holdings'),
              array('data' => $status,      'class' => 'xc-circ-status ' . $status_class)
            );
            $rows[$item['ItemId']] = array('data' => $data);
          }
          // order by ItemId
          krsort($rows);
          $rows = array_values($rows);
        }
      }
    }
  }
  
  // add classes and other information and modify rows if there may be missing
  // last cell, or the number of rows are greater than maximum displayable

  $count = count($rows);
  $hide_rows = $count > $max_rows_to_display;
     
  for ($i=0; $i < $count; $i++) {
    $rows[$i]['class'] = '';
    
    if ($i == 0) {
      $rows[$i]['class'] .= ' first';
    }
    elseif ($i == (count($rows) - 1)) {
      $rows[$i]['class'] .= ' last';
    }
    
    if ($i >= $max_rows_to_display) {
      $rows[$i]['class'] .= ' hidden';
    }
    
    if ($has_holdings_info && count($rows[$i]['data']) == 3) {
      $rows[$i]['data'][] = '';
    }
  }

  // Add a row, which contains a 'More' link to show all availability info
  if ($hide_rows) {
    $top_link = '<div id="xc-circ-link-on-top"><a href="#">' . t('Show less') . '</a></div>';
    $data = array(array(
      'data' => '<a href="#" id="xc-circ-link">'
              . t('Show more (!count)', array('!count' => $count)) . '</a>',
      'align' => 'right',
      'colspan' => ($has_holdings_info ? 4 : 3),
    ));
    $rows[] = array(
      'data' => $data,
      'id' => 'trigger-1'
    );
  }
  
  $header = array(t('Location'), t('Call number'), '', t('Availability'));
  if ($has_holdings_info) { 
    $header = array(t('Location'), t('Call number'), t('Volume information'), t('Availability'));
  }
  return array(
    'locations' => $top_link . theme('table', $header, $rows, array('class' => 'xc-circ')),
    'count' => $count
  );
}

function xc_search_ajax_more_facet() { // $facet, $query
  // xc_search/more_facets
  $query = $_GET['query'];
  $facet = $_GET['facet'];
  if (!preg_match('/^[a-z]\w+_fc$/', $facet)) {
    drupal_json(array(
      'status' => 0,
      'error_msg' => t('Sorry, %this is not a valid facet name.', array('%this' => $facet))
    ));
  }
  $index_facet = xc_index_facet_load_by_name(preg_replace('/_fc/', '', $facet));

  $filters = !empty($_GET['filter']) ? (array)$_GET['filter'] : array();
  $offset = 5;
  require_once drupal_get_path('module', 'xc_search') . '/searching/xc_search_block.inc';
  $checkbox_options = xc_search_block_checkbox_options();
  if ($checkbox_options['online_only'] == 1) {
    $filters[] = 'format_fc:"Online"';
  }
  if ($checkbox_options['exclude_microform'] == 1) {
    $filters[] = 'format_fc:"Not microform"';
  }

  $params = array(
    'facet' => 'on',
    'facet.field' => $facet,
    'facet.mincount' => 1,
    'facet.offset' => $offset,
    'facet.limit' => variable_get('xc_search_more_facet_inline', 15) + 1,
    'facet.missing' => 'on',
    'fq' => array_unique($filters),
  );

  $url_params = xc_search_get_params();
  unset($url_params['facet']);
  unset($url_params['query']);
  unset($url_params['page']);
  unset($url_params['offset']);
  
  $response = $GLOBALS['_xc_search_server']->search($query, 0, 0, $params, FALSE);
  $terms = array();
  $has_more = FALSE;
  if (isset($response->facet_counts->facet_fields)
      && isset($response->facet_counts->facet_fields->$facet)) {
    foreach ($response->facet_counts->facet_fields->$facet as $term => $count) {
      if ($count != 0 && $term != '_empty_') {
        $url_queries = array_merge_recursive($url_params, array('filter' => $facet . ':"' . $term . '"'));
        $items[] = '<span class="xc-search-facet-link">'
          . l('<span class="facet-term">' . $term . '</span>', 
              'xc/search/' . $query, array('query' => $url_queries, 'html' => TRUE)
            )
          . ' <span class="facet-term-count">(' . $count . ')</span>'
          . '</span>';
      }
    }
    if (count($items) >= $params['facet.limit']) {
      $has_more = TRUE;
      // remove one element from the list. This was just a check element whether it has more
      array_pop($items);
    }
    $output .= theme('item_list', $items);
  }
  else {
    drupal_json(array(
      'status' => 0, 
      'error_msg' => t('No terms in that facet'),
    ));
  }

  drupal_json(array(
    'status' => 1, 
    'content' => $output,
    'hasMore' => $has_more,
  ));
}

/**
 * Autocomplete function
 * 
 * @param $query (string)
 *   The string to get suggestions for
 */
function xc_search_autocomplete($query = '') {
  $suggestions = array();
  if ($query) {
    $suggestions = xc_search_suggestions($query, TRUE);
  }
  return drupal_json(drupal_map_assoc($suggestions));
}