<?php
/**
 * @file
 * Functions related to Solr indexing
 * 
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

define('XC_SOLR_SORT_STRATEGY_FIRST',  2);
define('XC_SOLR_SORT_STRATEGY_MINMAX', 1);

/**
 * Displays the schema field to Solr fields mapping
 */
function xc_solr_show_fields2index_list() {
  variable_del('xc_solr_fields2index_list');
  $list = xc_solr_fields2index_list();
  $rows = array();
  foreach ($list as $schema_field => $targets) {
    $row = array();
    $row[] = $schema_field;
    $output = array();
    foreach ($targets as $type => $solr_fields) {
      
      if ($type == 'value_as_key') {
        // array ( 'value_as_key' => array ( 'type' => array ( 'values' => array ( '@gtt ' => array ( 0 => 'xc__subject_gtt _t', ), ), ), ),
        foreach ($solr_fields as $attribute => $properties) {
          foreach ($properties['values'] as $property => $solr_fields2) {
            $output[] = '@' . $attribute . '=' . $property . ': ' . join(', ', $solr_fields2);
          }
        }
      }
      else {
        $output[] = $type . ': ' . join(', ', $solr_fields);
      }
    }
    $row[] = join('<br/>', $output);
      
    $rows[] = $row;
  }
  return theme('table', array(t('Schema field'), t('Solr fields')), $rows);
}

/**
 * Displays a confirmation form for launching the one-stop Solr indexer
 * 
 * @param $form_state (Array)
 *   The FAPI form state array
 */
function xc_solr_onestop_indexer_batch_launcher_form(&$form_state) {
  // TODO: check whether Solr is runing
  $form['xc_location'] = array(
    '#type' => 'value',
    '#value' => serialize($xc_location)
  );
  
  return confirm_form(
    array($form),
    t('Are you sure, that you would like to launch indexing metadata for search?'),
    'admin/xc/solr', // path to go if user click on 'cancel' 
    t('This action cannot be undone.'), 
    t('Index metadata'),
    t('Cancel')
  );
}

/**
 * Submission handler for the confirmation form for launching the one-stop Solr indexer.
 * 
 * @param $form (Array)
 *   The FAPI form array
 * @param $form_state (Array)
 *   The FAPI form state array
 */
function xc_solr_onestop_indexer_batch_launcher_form_submit($form, &$form_state) {
  global $_oaiharvester_statistics;
  
  $xc_location = unserialize($form_state['values']['xc_location']);
  variable_del('oaiharvester_statistics');
  variable_del('xc_solr_fields2index_list');
  $_oaiharvester_statistics = array();
  
  xc_solr_onestop_indexer_batch_launcher();
}

function xc_solr_onestop_indexer_batch_launcher() {
  drupal_set_message('xc_solr_onestop_indexer_batch_launcher');
  if (!function_exists('xc_location_get')) {
    include_once drupal_get_path('module', 'xc_metadata') . '/xc_metadata.location.inc';
  }
  variable_del('xc_solr_statistics_total');
  variable_set('xc_solr_fields2index_list_applied', xc_solr_fields2index_list('simple'));
  $locations = xc_location_get(1);
  $batch = array(
    'operations'       => array(array('xc_solr_onestop_indexer_batch', array(NULL, array($locations), -1, -1))),
    'title'            => t('Indexing Solr'),
    'init_message'     => t('Initialize reindexing process'),
    'progress_message' => t('Keep on running'),
    'finished'         => 'xc_solr_onestop_indexer_batch_finished',
  );
  batch_set($batch);
  batch_process('admin/xc/index');
}

function xc_solr_onestop_indexer($metadata_id) {
  global $_oaiharvester_statistics, $_xc_solr_conditional_facets;
  static $_xc_solr_field_hooks, $_xc_solr_sortable_fields;
  
  $_xc_solr_conditional_facets = array();
  // iterate manifestations
  $manifestation = xc_solr_get_metadata($metadata_id);
  $metadata = $manifestation->metadata;
  $document = xc_solr_metadata_to_solr_iterate_indexes($manifestation);
  
  // iterate holdings
  $holdings_ids = xc_entity_get_child_identifiers($manifestation->identifier_int);
  foreach ($holdings_ids as $holdings_identifier_int) {
    $holdings = xc_solr_get_metadata_by_identifier_int($holdings_identifier_int);
    if (is_array($holdings->metadata)) {
      $ft1 = microtime(TRUE);
      $metadata = array_merge_recursive($metadata, $holdings->metadata);
      $ft2 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/01 metadata merge'] += $ft2 - $ft1;
    }
    xc_solr_metadata_to_solr_iterate_indexes($holdings, $document);
  }
  
  // $expression_ids = xc_solr_get_parent($manifestation);
  $expressions = xc_solr_get_parent($manifestation);
  
  // foreach ($expression_ids as $expression_id) {
  foreach ($expressions as $expression) {
    // $expression = xc_solr_get_metadata($expression_id);
    if ($expression === FALSE) {
      xc_log_error('solr', 'empty expression in: ' . var_export($expressions, TRUE) . ', ' . var_export($manifestation->metadata, TRUE));
      continue;
    }
    if (is_array($expression->metadata)) {
      $ft1 = microtime(TRUE);
      $metadata = array_merge_recursive($metadata, $expression->metadata);
      $ft2 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/01 metadata merge'] += $ft2 - $ft1;
    }
    xc_solr_metadata_to_solr_iterate_indexes($expression, $document);
    // $work_ids = xc_solr_get_parent($expression);
    $works = xc_solr_get_parent($expression);
    
    // foreach ($work_ids as $work_id) {
    foreach ($works as $work) {
      // $work = xc_solr_get_metadata($work_id);
      if ($work === FALSE) {
        xc_log_error('solr', 'empty work in: ' . var_export($works, TRUE));
        continue;
      }
      if (is_array($work->metadata)) {
        $ft1 = microtime(TRUE);
        $metadata = array_merge_recursive($metadata, $work->metadata);
        $ft2 = microtime(TRUE);
        $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/01 metadata merge'] += $ft2 - $ft1;
      }
      xc_solr_metadata_to_solr_iterate_indexes($work, $document);
    }
  }
  $document->addField('metadata_tsn',  serialize($metadata), FALSE, TRUE);
  $document->setField('text', join(' ', $document->text));
  
  $ft1 = microtime(TRUE);
  
  // format facet (it is a special case of facets)
  // TODO: convert this to normal hook or DB based conditions
  xc_index_format_facet($document, $metadata);
  $ft2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/02 format facet'] += $ft2 - $ft1;
  
  // general case for facet hooks
  if (!empty($_xc_solr_conditional_facets)) {
    foreach ($_xc_solr_conditional_facets as $facet => $fields) {
      if (!isset($_xc_solr_field_hooks)) {
        $_xc_solr_field_hooks = variable_get('xc_solr_field_hooks', array());
      }
      if (!isset($_xc_solr_field_hooks[$facet])) {
        $_xc_solr_field_hooks[$facet] = array();
        $hook = 'xc_process_facet_' . $facet;
        foreach (module_implements($hook) as $module) {
          $_xc_solr_field_hooks[$facet][] = $module . '_' . $hook;
        }
        variable_set('xc_solr_field_hooks', $_xc_solr_field_hooks);
      }
      foreach ($_xc_solr_field_hooks[$facet] as $function) {
        $function($document, $metadata, array_keys($fields));
      }
    }
  }
  $ft3 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/03 conditional facets 1 - hooks'] += $ft3 - $ft2;
  
  // general case for conditional facet existing in database
  require_once drupal_get_path('module', 'xc_index') . '/xc_index_facet.inc';
  $conditional_facets = xc_index_facet_list_conditionals(TRUE);
  if (!empty($conditional_facets)) {
    foreach ($conditional_facets as $facet) {
      if ($facet->fn !== FALSE) {
        require_once drupal_get_path('module', 'xc_index') . '/xc_index_facet_mapping.inc';
        $mapping = xc_index_facet_for_edit_load($facet->name);
        $function = $facet->fn;
        $values = $function(&$document, $metadata, $mapping->fields);
      }
    }
  }
  $ft4 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/04 conditional facets 2 - forms'] += $ft4 - $ft3;
  
  if (!isset($_xc_solr_sortable_fields)) {
    if (!function_exists('xc_index_sortables_get_all')) {
      require_once drupal_get_path('module', 'xc_index') . '/xc_index_sortables.inc';
    }
    $fields = xc_index_sortables_get_all();
    if (!empty($fields)) {
      $suffixes = array_diff(array_keys(xc_solr_field_type_get_types()), array('n/a'));
      $regex = '/^(.*?)(' .  join('|', $suffixes) . ')$/';
      $_xc_solr_sortable_fields = array();
      foreach ($fields as $field) {
        if (preg_match($regex, $field, $matches)) {
          $_xc_solr_sortable_fields[$field] = array('name' => $matches[1], 'suffix' => $matches[2]);
        }
        else {
          $_xc_solr_sortable_fields[$field] = array('name' => $field, 'suffix' => '');
        }
      }
    }
    else {
      $_xc_solr_sortable_fields = array();
    }
    
    /*
    $_xc_solr_sortable_fields = array(
      'date_df' => array('name' => 'date', 'suffix' => '_df'),
      'dcterms__title_s' => array('name' => 'dcterms__title', 'suffix' => '_s'),
      'authors_contributors_fc' => array('name' => 'authors_contributors', 'suffix' => '_fc'),
    );
    */
  }
  
  if (!empty($_xc_solr_sortable_fields)) {
    $sort_strategy = xc_solr_get_sortstrategy();
    foreach ($_xc_solr_sortable_fields as $field => $properties) {
      if (isset($document->$field)) {
        $instances = $document->$field;
        if (!empty($instances)) {
          $suffix = ($properties['suffix'] == '_dt' || $properties['suffix'] == '_df') ? '_sdt' : '_ss';
          $len = count($instances) - 1;
          if ($sort_strategy == XC_SOLR_SORT_STRATEGY_MINMAX) {
            sort($instances);
            for ($i = 0; $i <= $len; $i++) {
              if ($instances[$i] != '') {
                $document->addField($properties['name'] . '_min' . $suffix, $instances[$i]);
                break;
              }
            }
            for ($i = $len; $i >= 0; $i--) {
              if ($instances[$i] != '') {
                $document->addField($properties['name'] . '_max' . $suffix, $instances[$i]);
                break;
              }
            }
          }
          elseif ($sort_strategy == XC_SOLR_SORT_STRATEGY_FIRST) {
            for ($i = 0; $i <= $len; $i++) {
              if ($instances[$i] != '') {
                $document->addField($properties['name'] . $suffix, $instances[$i]);
                break;
              }
            }
          }
        }
      }
    }
  }
  $ft5 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/05 sortable fields'] += $ft5 - $ft4;

  return $document;
}

function xc_solr_get_metadata($metadata_id) {
  global $_oaiharvester_statistics;
  $t = microtime(TRUE);
  if (empty($metadata_id)) {
    return FALSE;
  }
  // xc_log_info('solr', 'get #' . $metadata_id);
  $sql = 'SELECT ent.*, meta.metadata AS metadata FROM {xc_entity_properties} AS ent
    INNER JOIN {xc_sql_metadata} AS meta ON (ent.metadata_id = meta.metadata_id)
    WHERE ent.metadata_id = %d';
  $entity = db_fetch_object(db_query($sql, $metadata_id));
  $metadata = unserialize($entity->metadata);
  if ($metadata == FALSE) {
    xc_log_error('solr', 'Wrong serialization #' . $metadata_id . ': ' . $entity->metadata);
  }
  $entity->metadata = $metadata;
  $t2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/01 xc_solr_get_metadata'] += ($t2 - $t);
  
  //drupal_set_message(var_export($entity, TRUE));
  return $entity; // $source_id,
}

function xc_solr_get_metadata_by_identifier($identifier, $identifier_prefix) {
  $identifier_int = xc_util_get_identifier_int($identifier, $identifier_prefix);
  return xc_solr_get_metadata_by_identifier_int($identifier_int);
}

function xc_solr_get_metadata_by_identifier_int($identifier_int) {
  global $_oaiharvester_statistics, $_xc_solr_false_metadata;
  
  $t = microtime(TRUE);
  if (empty($identifier_int)) {
    $_xc_solr_false_metadata['empty']++;
    return FALSE;
  }
  // xc_log_info('solr', 'get #' . $metadata_id);
  $sql = 'SELECT ent.*, meta.metadata AS metadata FROM {xc_entity_properties} AS ent
    INNER JOIN {xc_sql_metadata} AS meta ON (ent.metadata_id = meta.metadata_id)
    WHERE ent.identifier_int = %d';
  $entity = db_fetch_object(db_query($sql, $identifier_int));
  if ($entity != FALSE) {
    $metadata = unserialize($entity->metadata);
    if ($metadata == FALSE) {
      $_xc_solr_false_metadata['metadata']++;
      xc_log_error('solr', 'Wrong serialization #' . $identifier . ' ('. $identifier_int . '): ' . $entity->metadata);
    }
    else {
      $_xc_solr_false_metadata['success']++;
    }
    $entity->metadata = $metadata;
  }
  else {
    $_xc_solr_false_metadata['entity']++;
  }
  $t2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/xc_solr_get_parent/xc_solr_get_metadata_by_identifier'] += ($t2 - $t);
  
  //drupal_set_message(var_export($entity, TRUE));
  return $entity; // $source_id,
}

function xc_solr_get_parent($entity) {
  global $_oaiharvester_statistics;
  $t_start = microtime(TRUE);
  $metadatas = array();
  
  // $parents = xc_entity_get_relationships($entity, 'parent', TRUE, FALSE, TRUE);
  $parents = xc_entity_get_parent_identifiers($entity->identifier_int);
  if ($parents) {
    foreach ($parents as $identifier_int) {
      // $metadata = xc_solr_get_metadata_by_identifier_int($identifier, $identifier_prefix);
      $metadata = xc_solr_get_metadata_by_identifier_int($identifier_int);
      if (empty($metadata)) {
        xc_log_error('solr', 'No metadata for ' . $identifier_int . ' (parent of an ' . $entity->metadata_type . ')');
      }
      else {
        $metadatas[] = $metadata;
      }
    }
  }
  else {
    if ($entity->metadata_type == 'manifestation') {
      $parents = $entity->metadata['xc:expressionManifested'];
    }
    elseif ($entity->metadata_type == 'expression') {
      $parents = $entity->metadata['xc:workExpressed'];
    }
    elseif ($entity->metadata_type == 'holdings') {
      $parents = $entity->metadata['xc:manifestationHeld'];
    }
    elseif ($entity->metadata_type == 'item') {
      $parents = $entity->metadata['xc:holdingsExemplified'];
    }
    if (!empty($parents)) {
      $identifier_prefix = _source_id_to_identifier_prefix($entity->source_id);
      foreach ($parents as $identifier) {
        // $metadata_id = _identifier_to_metadata_id($identifier['#value']);
        $metadata = xc_solr_get_metadata_by_identifier($identifier['#value'], $identifier_prefix);
        $t2 = microtime(TRUE);
        if (empty($metadata)) {
          xc_log_error('solr', 'No metadata for ' . $identifier['#value'] . ' (' . $entity->metadata_type . ')');
        }
        else {
          $metadatas[] = $metadata;
        }
      }
    }
  }
  $t_end = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/xc_solr_get_parent'] += ($t_end - $t_start);
  return $metadatas;
}
/**
 * 
 * @param $type
 * @return unknown_type
 */
function xc_solr_metadata_to_solr_factory(&$object, $type, &$document = NULL) {
  switch ($type) {
    case 'iterate over fields':
      return xc_solr_metadata_to_solr_iterate_fields($object, $document); break;
    case 'iterate over indexes':
      return xc_solr_metadata_to_solr_iterate_indexes($object, $document); break;
  }
}

/**
 * Converts a metadata object to Solr document
 * 
 * @param $object (Object)
 *   A metadata object. 
 *   an example:
 *   identifier => 'oai:URMST:Transformation Service/10002'
 *   (optional) 'node_id' => ...
 *   (optional) 'node_type' => ...
 *   metadata_id => ...,
 *   metadata_type => ...,
 *   source_id => ...
 *   metadata => array(
 *     "@type" => "expression",
 *     "@id"   => "oai:mst.rochester.edu:MST/MARCToXCTransformation/14999",
 *     "xc:titleOfExpression" = array(array("#value" => "Philosophy of art history.")),
 *     "dcterms:type" => array(array("@type" => "dcterms:DCMIType", "#value" => "Text")),
 *     ...
 *   );
 */
function xc_solr_metadata_to_solr_iterate_fields(&$object, &$document) {
  global $_oaiharvester_statistics;
  static $cache = array(
    'unique_fieldnames' => array('serialized_tsn'),
    'subject_topic_facet_field' => array(
      array('#name' => 'subject_fc', '#type' => 'facet'),
      array('#name' => 'topic_fc', '#type' => 'facet'),
      array('#name' => 'subject_topic_fc', '#type' => 'facet'),
  ));
  
  $t_new = microtime(TRUE);
  if (is_null($document)) {
    $document = new Apache_Solr_DocumentXc();
  }
  $document->setUniqueFieldnames($cache['unique_fieldnames']);
  $t_new2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/new'] += ($t_new2 - $t_new);
  
  $debug = FALSE;
  //$debug = ($object->identifier == 'oai:URMST:Transformation Service/10002') ? TRUE : FALSE;
  
  $document->addField('id',              $object->identifier, FALSE, TRUE);
  if (isset($object->node_id)) {
    $document->addField('node_id_i',     $object->node_id, FALSE, TRUE);
  }
  if (isset($object->node_type)) {
    $document->addField('node_type_s',   $object->node_type, FALSE, TRUE);
  }
  $document->addField('metadata_id_s',   $object->metadata_id, FALSE, TRUE);
  $document->addField('metadata_type_s', $object->metadata_type, FALSE, TRUE);
  $document->addField('source_id_s',     $object->source_id, FALSE, TRUE);
  $document->addField('serialized_tsn',  serialize($object), FALSE, TRUE);
  $t_new3 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/common fields'] += ($t_new3 - $t_new2);
  $content = '';
  foreach ($object->metadata as $field => $values) {
    if (!isset($values) || empty($values)) {
      continue;
    }
    
    $solr_fields = array();
    unset($solr_field);
    
    // set field name
    if ($field == '@type') {
      $solr_field = 'type';
    }
    elseif ($field == '@id' || $field == 'metadata_id' || $field == 'xc_id') {
      $solr_field = 'id';
    }
    else {
      // fieldnames
      $t = microtime(TRUE);
      foreach (xc_solr_get_type_suffixes($field) as $suffix) {
        if ($suffix == '_dt') {
          $solr_fields[] = array(
            '#name' => xc_solr_get_solr_field_name($field, $suffix), 
            '#type' => 'date_field'
          );
        }
        else {
          $solr_fields[] = xc_solr_get_solr_field_name($field, $suffix);
        }
      }
      $t2 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/solr fields'] += ($t2 - $t);
    }
    // delayed insert of type and id
    if (isset($solr_field)) {
      $solr_fields[] = array('#name' => $solr_field, '#type' => 'field');
    }
      
    // handle facet aliases
    $t = microtime(TRUE);
    $facets = xc_solr_get_facet_aliases($field);
    $t2 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/xc_solr_get_facet_aliases'] += ($t2 - $t);
    $solr_fields = array_merge($solr_fields, $facets);

    // handle values
    if (!is_array($values)) {
      if ($solr_field == 'type') {
        $document->addField($solr_field, $values, FALSE, TRUE);
      }
      elseif ($solr_field == 'id') {
        if (!$document->getField('id')) {
          $document->addField($solr_field, $values, FALSE, TRUE);
        }
      }
      else {
        drupal_set_message(t('not an array: %f=%v', 
          array('%f' => $field, '%v' => var_export($values, TRUE))));
      }
    }
    // handle values in an array
    else {
      foreach ($values as $element_instance) {
        // problem in metadata format
        if (!is_array($element_instance)) {
          xc_log_error('Solr', 'not an array: ' . var_export($element_instance, TRUE));
        }
        
        $actual_solr_fields = $solr_fields;
        foreach ($element_instance as $key => $value) {
          // field value
          if ($key == '#value') {
            $content .= $value . ' ';
            if (!empty($facets)) {
              $t = microtime(TRUE);
              $cleaned_value = _xc_solr_clean($value);
              $t2 = microtime(TRUE);
              $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/_xc_solr_clean'] += ($t2 - $t);
            }
            foreach ($actual_solr_fields as $solr_field) {
              if ($solr_field['#type'] == 'facet' && $cleaned_value != '') {
                _xc_solr_add_field($document, $solr_field, $cleaned_value);
              }
              elseif ($value != '') {
                _xc_solr_add_field($document, $solr_field, $value);
              }
            }
          }
          // handling attributes
          else {
            // remove subject's LCC, DDC
            if ($field == 'dcterms:subject' && $key = '@type' 
                && in_array($value, array('dcterms:LCC', 'dcterms:DDC'))) {
              // remove subject facet
              $t = microtime(TRUE);
              xc_util_remove_array_element($actual_solr_fields, $cache['subject_topic_facet_field']);
              $t2 = microtime(TRUE);
              $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/xc_util_remove_array_element'] += ($t2 - $t);
            }
            $t = microtime(TRUE);
            $result = xc_solr_process_field_attribute($field, $key, $value, $debug);
            $t2 = microtime(TRUE);
            $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/xc_solr_process_field_attribute'] += ($t2 - $t);
            if ($result) {
              if (count($result['as_key_exclusively']) > 0) {
                $actual_solr_fields = $result['as_key_exclusively'];
              }
              else {
                $actual_solr_fields = array_merge($actual_solr_fields, $result['value_as_key']);
              }
              if (!empty($result['value_as_value'])) {
                foreach ($result['value_as_value'] as $attr_field) {
                  if ($attr_field['#type'] == 'facet' && $cleaned_value != '') {
                    _xc_solr_add_field($document, $attr_field, $cleaned_value);
                  }
                  elseif ($value != '') {
                    _xc_solr_add_field($document, $attr_field, $value);
                  }
                }
              }
            }
          } // handling attributes
        } // foreach element_instances
      } // foreach values
    } // processing arrays
  } // processing metadata
  $field = 'text2';
  _xc_solr_add_field($document, $field, $content);
  return $document;
}

/**
 * Converts a metadata object to Solr document
 * 
 * @param $object (Object)
 *   A metadata object. 
 *   an example:
 *   identifier => 'oai:URMST:Transformation Service/10002'
 *   (optional) 'node_id' => ...
 *   (optional) 'node_type' => ...
 *   metadata_id => ...,
 *   metadata_type => ...,
 *   source_id => ...
 *   metadata => array(
 *     "@type" => "expression",
 *     "@id"   => "oai:mst.rochester.edu:MST/MARCToXCTransformation/14999",
 *     "xc:titleOfExpression" = array(array("#value" => "Philosophy of art history.")),
 *     "dcterms:type" => array(array("@type" => "dcterms:DCMIType", "#value" => "Text")),
 *     ...
 *   );
 */
function xc_solr_metadata_to_solr_iterate_indexes(&$object, &$document = NULL) {
  global $_oaiharvester_statistics, $_xc_solr_conditional_facets;
  
  //xc_log_info('solr', var_export($object, TRUE));
  static $cache = array(
    'unique_fieldnames' => array('serialized_tsn'),
    'subject_topic_facet_field' => array(
      array('#name' => 'subject_fc', '#type' => 'facet'),
      array('#name' => 'topic_fc', '#type' => 'facet'),
      array('#name' => 'subject_topic_fc', '#type' => 'facet'),
  ));
  $t_new = microtime(TRUE);
  if (is_null($document)) {
    $new_document = TRUE;
    $document = new Apache_Solr_DocumentXc();
  }
  else {
    $new_document = FALSE;
  }
  $document->setUniqueFieldnames($cache['unique_fieldnames']);
  $t_new2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/new'] += ($t_new2 - $t_new);
  
  $debug = FALSE;
  //$debug = ($object->identifier == 'oai:URMST:Transformation Service/10002') ? TRUE : FALSE;
  if ($new_document) { // $object->metadata_type == 'manifestation') {
    $document->addField('id',              $object->identifier, FALSE, TRUE);
    if (isset($object->node_id)) {
      $document->addField('node_id_i',     $object->node_id, FALSE, TRUE);
    }
    if (isset($object->node_type)) {
      $document->addField('node_type_s',   $object->node_type, FALSE, TRUE);
    }
    $document->addField('metadata_id_s',   $object->metadata_id, FALSE, TRUE);
    $document->addField('metadata_type_s', $object->metadata_type, FALSE, TRUE);
    $document->addField('source_id_s',     $object->source_id, FALSE, TRUE);
    // $document->addField('serialized_tsn',  serialize($object), FALSE, TRUE);
    $document->addField('type',            $object->metadata['@type'], FALSE, TRUE);
    if (!$document->getField('id')) {
      $document->addField('id',            $object->metadata['@id'], FALSE, TRUE);
    }
  }
  $t_new3 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/common fields'] += ($t_new3 - $t_new2);
  $indexes = xc_solr_fields2index_list();
  foreach ($indexes as $field => $field_properties) {
    if (!isset($object->metadata[$field])) {
      continue;
    }
    /*
    if (isset($field_properties['value_as_key']) || isset($field_properties['value_as_value'])) {
      drupal_set_message($field . ': ' . var_export($field_properties, TRUE));
    }
    */
    
    // main field value
    $tm = microtime(TRUE);
    if (isset($field_properties['general_fields']) && isset($object->metadata[$field])) {
      $values = array();
      foreach ($object->metadata[$field] as $instance) {
        $values[] = $instance['#value'];
      }
      xc_solr_add_fields($document, $field_properties['general_fields'], $values);
    }

    if (isset($field_properties['date_fields']) && isset($object->metadata[$field])) {
      $values = array();
      foreach ($object->metadata[$field] as $instance) {
        $temp = xc_solr_to_date($instance['#value'], $field);
        if (strlen($temp) == 20) {
          $values[] = $temp;
        }
      }
      xc_solr_add_fields($document, $field_properties['date_fields'], $values);
    }
    
    // use attribute value as field value
    $tm2 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/main value'] += ($tm2 - $tm);
    if (isset($field_properties['value_as_value'])) {
      foreach ($field_properties['value_as_value'] as $attribute) {
        if (isset($object->metadata[$field][$attribute])) {
          xc_solr_add_fields($document, $field_properties['value_as_value'][$attribute], $object->metadata[$field][$attribute]);
          xc_solr_add_fields($document, $field_properties['general_fields'], $object->metadata[$field][$attribute]);
        }
      }
    }
    
    // use attribute value as part of field name
    $tm3 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/value_as_value'] += ($tm3 - $tm2);
    if (isset($field_properties['value_as_key'])) {
      foreach ($field_properties['value_as_key'] as $attribute => $attribute_properties) {
        for ($len = count($object->metadata[$field]), $i = 0; $i < $len; $i++) {
          if (isset($object->metadata[$field][$i]['@' . $attribute])) {
            $key = $object->metadata[$field][$i]['@' . $attribute];
            if (isset($attribute_properties['values'][$key])) {
              xc_solr_add_fields($document, $attribute_properties['values'][$key], $object->metadata[$field][$i]['#value']);
            }
          }
        }
      }
    }
    
    // use attribute value as part of field name and do not use main field
    $tm4 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/value_as_value'] += ($tm4 - $tm3);
    // TODO: revision
    if (isset($field_properties['as_key_exclusively'])) {
      foreach ($field_properties['as_key_exclusively'] as $attribute) {
        $values = array();
        foreach ($object->metadata[$field] as $instance) {
          if (isset($instance[$attribute])) {
            $values[] = $instance[$attribute];
          }
        }
        if (!empty($values)) {
          xc_solr_add_fields($document, $field_properties['as value'][$attribute], $values);
        }
      }
    }
    
    // date facet
    $tm5 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/as_key_exclusively'] += ($tm5 - $tm4);
    if (isset($field_properties['date_facets'])) {
      $values = array();
      foreach ($object->metadata[$field] as $instance) {
        $temp = xc_solr_to_date($instance['#value'], $field);
        if (strlen($temp) == 20) {
          $values[] = $temp;
        }
      }
      xc_solr_add_fields($document, $field_properties['date_facets'], $values);
    }
    
    // facet
    $tm6 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/date_facet'] += ($tm6 - $tm5);
    if (isset($field_properties['facets'])) {
      $values = array();
      $tm61 = microtime(TRUE);
      foreach ($object->metadata[$field] as $instance) {
        $values[] = _xc_solr_clean($instance['#value']);
      }
      $tm62 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/facet/get values'] += ($tm62 - $tm61);
      xc_solr_add_fields($document, $field_properties['facets'], $values);
      $tm63 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/facet/add fields'] += ($tm63 - $tm62);
    }
    $tm7 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/facet'] += ($tm7 - $tm6);
    
    // collect conditional facets. We will run all this at the end of processing
    if (isset($field_properties['conditional_facets'])) {
      foreach ($field_properties['conditional_facets'] as $facet) {
        if (!isset($_xc_solr_conditional_facets[$facet])) {
          $_xc_solr_conditional_facets[$facet] = array();
        }
        if (!isset($_xc_solr_conditional_facets[$facet][$field])) {
          $_xc_solr_conditional_facets[$facet][$field] = 1;
        }
      }
    }
  }
  
  // text field, which contains everything
  $tm1 = microtime(TRUE);
  $content = '';
  if (is_array($object->metadata)) {
    foreach ($object->metadata as $field => $values) {
      if (!isset($values) || empty($values) || !is_array($values)) {
        continue;
      }
      foreach ($values as $element_instance) {
        foreach ($element_instance as $key => $value) {
          if ($key == '#value') {
            $content .= $value . ' ';
          } // foreach element_instances
        } // foreach values
      } // processing arrays
    }
  } // processing metadata
  $document->addField('text', $content, FALSE, TRUE);
  $t2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr/text'] += ($t2 - $tm1);
  $_oaiharvester_statistics['03 step3/iterator/01 xc_solr_onestop_indexer/toSolr'] += ($t2 - $t_new);
  return $document;
}

function xc_solr_add_fields(&$document, $fields, $values, $function = NULL) {
  if (!is_array($values)) {
    $values = (array)$values;
  }
  if (!is_null($function)) {
    array_walk($value, $function);
  }
  foreach ($fields as $field) {
    $document->addField($field, $values, FALSE, TRUE);
  }
}

function xc_solr_get_sortstrategy() {
  return XC_SOLR_SORT_STRATEGY_MINMAX;
}